{******************************************************************}
{ LCD Control                                                      }
{                                                                  }
{ Unit for controlling a LCD display (4x20 chars) controlled       }
{ by a HD44780 in 8-bit mode                                       }
{******************************************************************}
{ 1999 by Hans Luyten                                              }
{ Thanks to Randy Rasa (I did some snooping in his C-code)         }
{******************************************************************}

unit lcdctrl;

interface

uses crt;

type LCDChar = array [1..8] of byte;

const LPT       = $378;
      LCD_Data  = LPT;
      LCD_CTRL  = LPT+2;

procedure LCD_WriteCTRL(data : byte);
procedure LCD_WriteDATA(data : byte);
procedure LCD_Clear;
procedure LCD_Home;
procedure LCD_WriteChar(data : char);
procedure LCD_Write(data : string);
procedure LCD_InitDriver;
procedure LCD_Init;
procedure LCD_Goto(x,y : byte);
procedure LCD_WritePos(x,y : byte; data : string);
procedure LCD_WriteRow(row : byte; data :string);
procedure LCD_DefineChar(CharNo : byte;Data : LCDChar);

implementation

procedure LCD_WriteCTRL(data : byte);
begin
  port[LCD_CTRL]:=3;
  port[LCD_Data]:=data;
  port[LCD_CTRL]:=2;
  port[LCD_CTRL]:=3;
  port[LCD_CTRL]:=1;
  delay(10);
end;

procedure LCD_WriteDATA(data : byte);
begin
  port[LCD_CTRL]:=7;
  port[LCD_Data]:=data;
  port[LCD_CTRL]:=6;
  port[LCD_CTRL]:=7;
  port[LCD_CTRL]:=5;
  delay(1);
end;

procedure LCD_Clear;
begin
  LCD_WriteCTRL($1);
end;

procedure LCD_Home;
begin
  LCD_WriteCTRL($2);
end;

procedure LCD_WriteChar(data : char);
begin
  LCD_WriteDATA(ord(data));
end;

procedure LCD_Write(data : string);
var counter : integer;
begin
  for counter:=1 to length(data) do
    LCD_WriteChar(data[counter]);
end;

procedure LCD_InitDriver;
begin
  LCD_WriteCTRL($38);
  LCD_WriteCTRL($38);
  LCD_WriteCTRL($38);
  LCD_WriteCTRL($6);
  LCD_WriteCTRL($c);
end;

procedure LCD_Init;
begin
  LCD_InitDriver;
  LCD_Clear;
  LCD_Home;
end;

procedure LCD_Goto(x,y : byte);
var counter : integer;
begin
  case y of
   1: LCD_WriteCTRL($80+x-1);
   2: LCD_WriteCTRL($c0+x-1);
   3: LCD_WriteCTRL($94+x-1);
   4: LCD_WriteCTRL($d4+x-1);
   end;
end;


procedure LCD_WritePos(x,y : byte; data : string);
begin
  LCD_Goto(x,y);
  LCD_Write(data);
end;

procedure LCD_WriteRow(row : byte; data :string);
begin
  LCD_Goto(1,row);
  LCD_Write(data);
end;

procedure LCD_DefineChar(CharNo : byte;Data : LCDChar);
var counter:integer;
begin
  LCD_WriteCTRL($40 + (CharNo SHL 3));
  for counter:= 1 to 8 do
    LCD_WriteDATA(Data[Counter]);

end;

end.
